
From: Deepak Saxena <dsaxena@plexity.net>

Version 6 of the ARM architecture introduces the concept of 16MB pages
(supersections) and 36-bit (40-bit actually, but nobody uses this) physical
addresses.  36-bit addressed memory and I/O and ARMv6 can only be mapped
using supersections and the requirement on these is that both virtual and
physical addresses be 16MB aligned.  In trying to add support for ioremap()
of 36-bit I/O, we run into the issue that get_vm_area() allows for a
maximum of 512K alignment via the IOREMAP_MAX_ORDER constant.  To work
around this, we can:

- Allocate a larger VM area than needed (size + (1ul << IOREMAP_MAX_ORDER))
  and then align the pointer ourselves, but this ends up with 512K of 
  wasted VM per ioremap().

- Provide a new __get_vm_area_aligned() API and make __get_vm_area() sit
  on top of this. I did this and it works but I don't like the idea 
  adding another VM API just for this one case.

- My preferred solution which is to allow the architecture to override
  the IOREMAP_MAX_ORDER constant with it's own version. 

Signed-off-by: Deepak Saxena <dsaxena@plexity.net>
Cc: Russell King <rmk@arm.linux.org.uk>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 include/linux/vmalloc.h |    8 ++++++++
 mm/vmalloc.c            |    2 --
 2 files changed, 8 insertions(+), 2 deletions(-)

diff -puN include/linux/vmalloc.h~arm-allow-for-arch-specific-ioremap_max_order include/linux/vmalloc.h
--- 25/include/linux/vmalloc.h~arm-allow-for-arch-specific-ioremap_max_order	Mon Aug 29 15:03:30 2005
+++ 25-akpm/include/linux/vmalloc.h	Mon Aug 29 15:03:30 2005
@@ -10,6 +10,14 @@
 #define VM_MAP		0x00000004	/* vmap()ed pages */
 /* bits [20..32] reserved for arch specific ioremap internals */
 
+/*
+ * Maximum alignment for ioremap() regions.
+ * Can be overriden by arch-specific value.
+ */
+#ifndef IOREMAP_MAX_ORDER
+#define IOREMAP_MAX_ORDER	(7 + PAGE_SHIFT)	/* 128 pages */
+#endif
+
 struct vm_struct {
 	void			*addr;
 	unsigned long		size;
diff -puN mm/vmalloc.c~arm-allow-for-arch-specific-ioremap_max_order mm/vmalloc.c
--- 25/mm/vmalloc.c~arm-allow-for-arch-specific-ioremap_max_order	Mon Aug 29 15:03:30 2005
+++ 25-akpm/mm/vmalloc.c	Mon Aug 29 15:03:30 2005
@@ -158,8 +158,6 @@ int map_vm_area(struct vm_struct *area, 
 	return err;
 }
 
-#define IOREMAP_MAX_ORDER	(7 + PAGE_SHIFT)	/* 128 pages */
-
 struct vm_struct *__get_vm_area(unsigned long size, unsigned long flags,
 				unsigned long start, unsigned long end)
 {
_
